@@ -637,7 +637,8 @@ def reverse_dictionary(the_dict):
     result = {}
     for key, value in the_dict.iteritems():
         if value in result:
-            raise RuntimeError, "non-reversible mapping"
+            raise RuntimeError(
+                    "non-reversible mapping, duplicate key '%s'" % value)
         result[value] = key
     return result
 
@@ -649,7 +650,7 @@ def set_sum(set_iterable):
 
 
 def div_ceil(nr, dr):
-    return (nr + dr -1) // dr
+    return -(-nr // dr)
 
 
 
@@ -1066,6 +1067,68 @@ def get_write_to_map_from_permutation(original, permuted):
 
 # }}}
 
+# {{{ graph algorithms
+
+def a_star(initial_state, goal_state, neighbor_map,
+        estimate_remaining_cost=None,
+        get_step_cost=lambda x, y: 1):
+    """
+    With the default cost and heuristic, this amounts to Dijkstra's algorithm.
+    """
+
+    from heapq import heappop, heappush
+
+    if estimate_remaining_cost is None:
+        def estimate_remaining_cost(x):
+            if x != goal_state:
+                return 1
+            else:
+                return 0
+
+    class AStarNode(object):
+        __slots__ = ["state", "parent", "path_cost"]
+        def __init__(self, state, parent, path_cost):
+            self.state = state
+            self.parent = parent
+            self.path_cost = path_cost
+
+    inf = float("inf")
+    init_remcost = estimate_remaining_cost(initial_state)
+    assert init_remcost != inf
+
+    queue = [(init_remcost, AStarNode(initial_state, parent=None, path_cost=0))]
+    visited_states = set()
+
+    while len(queue):
+        _, top = heappop(queue)
+        visited_states.add(top.state)
+
+        if top.state == goal_state:
+            result = []
+            it = top
+            while it is not None:
+                result.append(it.state)
+                it = it.parent
+            return result[::-1]
+
+        for state in neighbor_map[top.state]:
+            if state in visited_states:
+                continue
+
+            remaining_cost = estimate_remaining_cost(state)
+            if remaining_cost == inf:
+                continue
+            step_cost = get_step_cost(top, state)
+
+            estimated_path_cost = top.path_cost+step_cost+remaining_cost
+            heappush(queue, 
+                (estimated_path_cost, 
+                    AStarNode(state, top, path_cost=top.path_cost + step_cost)))
+
+    raise RuntimeError("no solution")
+
+# }}}
+
 # {{{ formatting --------------------------------------------------------------
 # {{{ table formatting --------------------------------------------------------
 class Table:
@@ -1290,6 +1353,37 @@ def typedump(val, max_seq=5, special_handlers={}):
         except TypeError:
             return val.__class__.__name__
 
+
+
+
+def invoke_editor(s, filename="edit.txt", descr="the file"):
+    from tempfile import mkdtemp
+    tempdir = mkdtemp()
+
+    from os.path import join
+    full_name = join(tempdir, filename)
+
+    outf = open(full_name, "w")
+    outf.write(str(s))
+    outf.close()
+
+    import os
+    if "EDITOR" in os.environ:
+        from subprocess import Popen
+        p = Popen([os.environ["EDITOR"], full_name])
+        os.waitpid(p.pid, 0)[1]
+    else:
+        print "(Set the EDITOR environment variable to be " \
+                "dropped directly into an editor next time.)"
+        raw_input("Edit %s at %s now, then hit [Enter]:"
+                % (descr, full_name))
+
+    inf = open(full_name, "r")
+    result = inf.read()
+    inf.close()
+
+    return result
+
 # }}}
 
 # {{{ progress bars -----------------------------------------------------------
